@@ -7,10 +7,9 @@
 
 -}
 
-module BNFC.Backend.CPP.STL (makeCppStl,) where
+module BNFC.Backend.CPP.STL (makeCppStl, mkEntryFuncs) where
 
 import Data.Foldable (toList)
-import qualified Data.Map as Map
 
 import BNFC.Utils
 import BNFC.CF
@@ -38,7 +37,7 @@ makeCppStl opts cf = do
     mkCppFileWithHint (name ++ ".l") flex
     mkCppFileWithHint (name ++ ".y") $ cf2Bison (linenumbers opts) parserMode cf env
     mkCppFile "Parser.H" $
-      mkHeaderFile (inPackage opts) cf (allParserCats cf) (toList $ allEntryPoints cf) (Map.elems env)
+      mkHeaderFile (inPackage opts) (toList $ allEntryPoints cf)
     mkCppFile "ParserError.H" $ printParseErrHeader (inPackage opts)
     let (skelH, skelC) = cf2CVisitSkel True (inPackage opts) cf
     mkCppFile "Skeleton.H" skelH
@@ -47,7 +46,7 @@ makeCppStl opts cf = do
     mkCppFile "Printer.H" prinH
     mkCppFile "Printer.C" prinC
     mkCppFile "Test.C" (cpptest (inPackage opts) cf)
-    Makefile.mkMakefile opts $ makefile prefix name
+    Makefile.mkMakefile (optMake opts) $ makefile prefix name
   where
     name :: String
     name = lang opts
@@ -165,7 +164,8 @@ cpptest inPackage cf = unlines $ concat
    def = identCat cat
    scope = nsScope inPackage
 
-mkHeaderFile inPackage _cf _cats eps _env = unlines $ concat
+mkHeaderFile :: Maybe String -> [Cat] -> String
+mkHeaderFile inPackage eps = unlines $ concat
   [ [ "#ifndef " ++ hdef
     , "#define " ++ hdef
     , ""
@@ -176,7 +176,7 @@ mkHeaderFile inPackage _cf _cats eps _env = unlines $ concat
     , ""
     , nsStart inPackage
     ]
-  , concatMap mkFuncs eps
+  , concatMap mkEntryFuncs eps
   , [ nsEnd inPackage
     , ""
     , "#endif"
@@ -184,7 +184,9 @@ mkHeaderFile inPackage _cf _cats eps _env = unlines $ concat
   ]
   where
   hdef = nsDefine inPackage "PARSER_HEADER_FILE"
-  mkFuncs s =
+
+mkEntryFuncs :: Cat -> [String]
+mkEntryFuncs s =
     [ identCat (normCat s) ++ "*" +++ "p" ++ identCat s ++ "(FILE *inp);"
     , identCat (normCat s) ++ "*" +++ "p" ++ identCat s ++ "(const char *str);"
     ]
